# Generated by Django 3.2.16 on 2025-02-06 19:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('course_flow', '0116_auto_20230913_1423'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='notification',
            options={'ordering': ['-created_on']},
        ),
    ]
