# Generated by Django 2.2.25 on 2022-03-21 18:02

from django.db import migrations


class Migration(migrations.Migration):

    atomic = False

    dependencies = [
        ("course_flow", "0080_auto_20220308_1956"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="CustomTerm",
            new_name="ObjectSet",
        ),
    ]
