# Generated by Django 2.2.25 on 2022-07-04 18:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0085_nodelink_text_position"),
    ]

    operations = [
        migrations.AddField(
            model_name="workflow",
            name="static_view",
            field=models.BooleanField(default=False),
        ),
    ]
