# Generated by Django 2.2.16 on 2021-04-01 17:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0054_auto_20210331_1748"),
    ]

    operations = [
        migrations.DeleteModel(
            name="OutcomeWorkflow",
        ),
    ]
