# Generated by Django 3.2.15 on 2022-11-22 15:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0096_auto_20221118_1539"),
    ]

    operations = [
        migrations.AddField(
            model_name="userassignment",
            name="completed_on",
            field=models.DateTimeField(null=True),
        ),
    ]
