{% extends "course_flow/base.html" %}
<!-- -->
{% load static i18n %}
<!-- -->
{% block header %}{% endblock %}
<!-- -->
{% block title %}{% trans "My Favourites" %} - CourseFlow{% endblock %}
<!-- -->
{% block metadescription %} {% endblock %}
<!-- -->
{% block body %}
<!-- -->
{% endblock %}
<!-- -->
{% block foot %} {% endblock %}
<!-- -->
{% csrf_token %}
<!-- -->
{% block read_only_scripts %}

{% endblock %}
<!-- -->
{% block scripts %} {{ block.super }}
<script nonce="{{request.csp_nonce}}">

  const iconpath = "{% static 'course_flow/img/images_svg/' %}";
  
  
  $(".menubar").removeClass("hidden");

  var library_renderer;
  const user_id = {{ user.id }};

  window.addEventListener("load",function(){
      favourites_renderer = new library_renderers.FavouritesRenderer();
      $(document).ajaxError(renderers.fail_function);
      favourites_renderer.render($("#container"));
      makeActiveSidebar("#my-favourites");
  });
</script>
{% endblock %}
