{% extends "course_flow/base.html" %}
<!-- -->
{% load static i18n %}
<!-- -->
{% block title %}{% trans "New Activity" %} - CourseFlow{% endblock %}
<!-- -->
{% block metadescription %} {% endblock %}
<!-- -->
{% block header %}{% endblock %}
<!-- -->
{% block body %}
<div id="create">
    <div class="workflow-create-wrapper">
      <h1>
        {% trans 'Create an activity:' %}
      </h1>
      <form method="post">
        {% csrf_token %} {{ form.as_p }}
        <input id="save-button" type="submit" value="Save" />
      </form>
    </div>
</div>
{% endblock %}
<!-- -->
{% block foot %} {% endblock %}
<!-- -->
{% csrf_token %}
<!-- -->
{% block scripts %} {{ block.super }}
<script nonce="{{request.csp_nonce}}">
  
  $("#create").detach().appendTo("#container");
</script>
{% endblock %}
