# Generated by Django 3.2.15 on 2023-09-13 14:23

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0115_notification_is_unread"),
    ]

    operations = [
        migrations.AddField(
            model_name="notification",
            name="comment",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="notifications",
                to="course_flow.comment",
            ),
        ),
        migrations.AlterField(
            model_name="notification",
            name="text",
            field=models.TextField(blank=True, null=True),
        ),
    ]
